<?php

namespace Drupal\llm_provider\Form;

use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\llm_provider\Config\ModelConfigLoader;
use Drupal\llm_provider\Enum\Bundles;
use Drupal\llm_provider\Utility\StringUtility;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form to configure known 3rd party providers.
 */
class LLMProviderSettingsForm extends ConfigFormBase {

  /**
   * Array of supported bundles and corresponding settings.
   *
   * @var array
   */
  protected array $supportedBundles = [];

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * The typed config service.
   *
   * @var \Drupal\Core\Config\TypedConfigManagerInterface
   */
  protected $typedConfigManager;

  /**
   * Module configurations.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected Config $configurations;

  /**
   * Module schema.
   *
   * @var array
   */
  protected array $configSchema;

  /**
   * Constructs a new AutoGeneratedConfigForm.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typed_config_manager
   *   The config manager service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    ModuleHandlerInterface $module_handler,
    TypedConfigManagerInterface $typed_config_manager
  ) {
    parent::__construct($config_factory);
    $this->moduleHandler = $module_handler;
    $this->typedConfigManager = $typed_config_manager;
    $this->configSchema = $this->typedConfigManager->getDefinition('llm_provider.settings');
    $this->configurations = $this->config('llm_provider.settings');
    $this->supportedModulesEnabled();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('module_handler'),
      $container->get('config.typed')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['llm_provider.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'llm_provider_settings_form';
  }

  /**
   * Populates supported bundles. Returns true if it does exist.
   */
  protected function supportedModulesEnabled(): void {
    $supported_settings = ModelConfigLoader::loadConfigSettings();
    foreach ($supported_settings as $module => $settings) {
      if ($this->moduleHandler->moduleExists($module)) {
        foreach ($settings['bundles'] as $bundle => $configuration) {
          $this->supportedBundles[$bundle][$module] = [
            'title' => $settings['title'],
            'configuration' => $configuration,
          ];
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Exception
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    // Nothing to do if no supported module is enabled.
    if (empty($this->supportedBundles)) {
      $form['message'] = [
        '#markup' => $this->t('No supported contributed LLM provider available.'),
      ];
      return $form;
    }

    $form['tabs'] = [
      '#type' => 'vertical_tabs',
    ];

    $mapping = [];

    foreach ($this->supportedBundles as $bundle => $modules) {
      $form[$bundle] = [
        '#type' => 'details',
        '#title' => Bundles::fromName(StringUtility::snakeToPascal($bundle))->value,
        '#group' => 'tabs',
      ];
      foreach ($modules as $module => $settings) {
        $form[$bundle][$module] = [
          '#type' => 'details',
          '#title' => $settings['title'],
          '#open' => TRUE,
        ];
        $specific_schema = $this->getNestedSchema($this->configSchema, explode('.', $settings['configuration']['settings_key']));
        $this->generateFormElements($form[$bundle][$module], $mapping, $specific_schema, $settings['configuration']['settings_key']);
      }
    }

    $form['#field_mapping'] = $mapping;
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $values = $form_state->getValues();
    $field_mapping = $form['#field_mapping'];
    foreach ($field_mapping as $hashed_key => $config_path) {
      $this->configurations->set($config_path . '.default', $values[$hashed_key]);
    }

    $this->configurations->save();
    parent::submitForm($form, $form_state);
  }

  /**
   * Helper function to generate form elements from schema.
   *
   * @param array $form
   *   The form.
   * @param array $mapping
   *   Mapping between hashed widget names and configuration paths.
   * @param array $schema
   *   Configuration schema of the provider.
   * @param string $config_path
   *   Path of the configuration values.
   * @param string $parent_hash
   *   Hashad name of the parent widget.
   */
  private function generateFormElements(array &$form, array &$mapping, array $schema, string $config_path, string $parent_hash = ''): void {
    foreach ($schema['mapping'] as $key => $definition) {
      $full_config_path = $config_path . '.' . $key;

      if (empty($parent_hash)) {
        $hash = md5($full_config_path);
        $mapping[$hash] = $full_config_path;
      }
      else {
        $hash = $parent_hash;
      }

      if (isset($definition['type']) && $definition['type'] == 'mapping') {
        $form[$hash]['#title'] = $definition['label'];
        $this->generateFormElements($form, $mapping, $definition, $full_config_path, $hash);
      }
      else {
        switch ($key) {
          case 'description':
            $form[$hash]['#description'] = $this->configurations->get($full_config_path);
            break;

          case 'type':
            $form[$hash]['#type'] = $this->mapSchemaTypeToFormType($this->configurations->get($full_config_path));
            if ($this->configurations->get($full_config_path) == 'bool') {
              unset($form[$hash]['#title']);
            }
            break;

          case 'default':
            $form[$hash]['#default_value'] = $this->configurations->get($full_config_path);
            break;

          case 'required':
            $form[$hash]['#required'] = $this->configurations->get($full_config_path);
            break;
        }
      }
    }
  }

  /**
   * Maps schema data types to form element types.
   *
   * @param string $type
   *   Data type of a configuration value.
   *
   * @return string
   *   Type of widget.
   */
  private function mapSchemaTypeToFormType(string $type): string {
    $map = [
      'string' => 'textfield',
      'bool' => 'checkbox',
    ];
    return $map[$type] ?? 'textfield';
  }

  /**
   * Return nested schema.
   *
   * @param array $schema
   *   The full tree of the Provider configuration schema.
   * @param array $path_elements
   *   Keys in configuration schema.
   *
   * @return mixed
   *   Schema for given path.
   *
   * @throws \Exception
   */
  private function getNestedSchema(array $schema, array $path_elements): mixed {
    foreach ($path_elements as $element) {
      if (isset($schema['mapping'][$element])) {
        $schema = $schema['mapping'][$element];
      }
      else {
        throw new \Exception("Schema path element not found: " . $element);
      }
    }
    return $schema;
  }

}
